@@ -1,4 +1,4 @@
-from typing import Optional, List, Union
+from typing import Optional, List, Type, Union
 import asyncio
 
 from deepeval.utils import get_or_create_event_loop, prettify_list
@@ -11,7 +11,6 @@ from deepeval.metrics.utils import (
 from deepeval.test_case import (
     LLMTestCase,
     LLMTestCaseParams,
-    ConversationalTestCase,
 )
 from deepeval.metrics import BaseMetric
 from deepeval.models import DeepEvalBaseLLM
@@ -21,14 +20,13 @@ from deepeval.metrics.contextual_relevancy.template import (
 from deepeval.metrics.indicator import metric_progress_indicator
 from deepeval.metrics.contextual_relevancy.schema import *
 
-required_params: List[LLMTestCaseParams] = [
-    LLMTestCaseParams.INPUT,
-    LLMTestCaseParams.ACTUAL_OUTPUT,
-    LLMTestCaseParams.RETRIEVAL_CONTEXT,
-]
-
 
 class ContextualRelevancyMetric(BaseMetric):
+    _required_params: List[LLMTestCaseParams] = [
+        LLMTestCaseParams.INPUT,
+        LLMTestCaseParams.RETRIEVAL_CONTEXT,
+    ]
+
     def __init__(
         self,
         threshold: float = 0.5,
@@ -37,6 +35,9 @@ class ContextualRelevancyMetric(BaseMetric):
         async_mode: bool = True,
         strict_mode: bool = False,
         verbose_mode: bool = False,
+        evaluation_template: Type[
+            ContextualRelevancyTemplate
+        ] = ContextualRelevancyTemplate,
     ):
         self.threshold = 1 if strict_mode else threshold
         self.model, self.using_native_model = initialize_model(model)
@@ -45,22 +46,29 @@ class ContextualRelevancyMetric(BaseMetric):
         self.async_mode = async_mode
         self.strict_mode = strict_mode
         self.verbose_mode = verbose_mode
+        self.evaluation_template = evaluation_template
 
     def measure(
         self,
-        test_case: Union[LLMTestCase, ConversationalTestCase],
+        test_case: LLMTestCase,
         _show_indicator: bool = True,
+        _in_component: bool = False,
     ) -> float:
-        if isinstance(test_case, ConversationalTestCase):
-            test_case = test_case.turns[0]
-        check_llm_test_case_params(test_case, required_params, self)
+
+        check_llm_test_case_params(test_case, self._required_params, self)
 
         self.evaluation_cost = 0 if self.using_native_model else None
-        with metric_progress_indicator(self, _show_indicator=_show_indicator):
+        with metric_progress_indicator(
+            self, _show_indicator=_show_indicator, _in_component=_in_component
+        ):
             if self.async_mode:
                 loop = get_or_create_event_loop()
                 loop.run_until_complete(
-                    self.a_measure(test_case, _show_indicator=False)
+                    self.a_measure(
+                        test_case,
+                        _show_indicator=False,
+                        _in_component=_in_component,
+                    )
                 )
             else:
                 self.verdicts_list: List[ContextualRelevancyVerdicts] = [
@@ -78,22 +86,23 @@ class ContextualRelevancyMetric(BaseMetric):
                     ],
                 )
 
-                return self.score
+            return self.score
 
     async def a_measure(
         self,
-        test_case: Union[LLMTestCase, ConversationalTestCase],
+        test_case: LLMTestCase,
         _show_indicator: bool = True,
+        _in_component: bool = False,
     ) -> float:
-        if isinstance(test_case, ConversationalTestCase):
-            test_case = test_case.turns[0]
-        check_llm_test_case_params(test_case, required_params, self)
+
+        check_llm_test_case_params(test_case, self._required_params, self)
 
         self.evaluation_cost = 0 if self.using_native_model else None
         with metric_progress_indicator(
             self,
             async_mode=True,
             _show_indicator=_show_indicator,
+            _in_component=_in_component,
         ):
             self.verdicts_list: List[ContextualRelevancyVerdicts] = (
                 await asyncio.gather(
@@ -120,28 +129,34 @@ class ContextualRelevancyMetric(BaseMetric):
         if self.include_reason is False:
             return None
 
-        irrelevancies = []
+        irrelevant_statements = []
         relevant_statements = []
         for verdicts in self.verdicts_list:
             for verdict in verdicts.verdicts:
                 if verdict.verdict.lower() == "no":
-                    irrelevancies.append(verdict.reason)
+                    irrelevant_statements.append(verdict.reason)
                 else:
                     relevant_statements.append(verdict.statement)
 
-        prompt: dict = ContextualRelevancyTemplate.generate_reason(
+        prompt: dict = self.evaluation_template.generate_reason(
             input=input,
-            irrelevancies=irrelevancies,
+            irrelevant_statements=irrelevant_statements,
             relevant_statements=relevant_statements,
             score=format(self.score, ".2f"),
         )
         if self.using_native_model:
-            res, cost = await self.model.a_generate(prompt, schema=Reason)
+            res, cost = await self.model.a_generate(
+                prompt, schema=ContextualRelevancyScoreReason
+            )
             self.evaluation_cost += cost
             return res.reason
         else:
             try:
-                res: Reason = await self.model.a_generate(prompt, schema=Reason)
+                res: ContextualRelevancyScoreReason = (
+                    await self.model.a_generate(
+                        prompt, schema=ContextualRelevancyScoreReason
+                    )
+                )
                 return res.reason
             except TypeError:
                 res = await self.model.a_generate(prompt)
@@ -152,28 +167,32 @@ class ContextualRelevancyMetric(BaseMetric):
         if self.include_reason is False:
             return None
 
-        irrelevancies = []
+        irrelevant_statements = []
         relevant_statements = []
         for verdicts in self.verdicts_list:
             for verdict in verdicts.verdicts:
                 if verdict.verdict.lower() == "no":
-                    irrelevancies.append(verdict.reason)
+                    irrelevant_statements.append(verdict.reason)
                 else:
                     relevant_statements.append(verdict.statement)
 
-        prompt: dict = ContextualRelevancyTemplate.generate_reason(
+        prompt: dict = self.evaluation_template.generate_reason(
             input=input,
-            irrelevancies=irrelevancies,
+            irrelevant_statements=irrelevant_statements,
             relevant_statements=relevant_statements,
             score=format(self.score, ".2f"),
         )
         if self.using_native_model:
-            res, cost = self.model.generate(prompt, schema=Reason)
+            res, cost = self.model.generate(
+                prompt, schema=ContextualRelevancyScoreReason
+            )
             self.evaluation_cost += cost
             return res.reason
         else:
             try:
-                res: Reason = self.model.generate(prompt, schema=Reason)
+                res: ContextualRelevancyScoreReason = self.model.generate(
+                    prompt, schema=ContextualRelevancyScoreReason
+                )
                 return res.reason
             except TypeError:
                 res = self.model.generate(prompt)
@@ -198,7 +217,7 @@ class ContextualRelevancyMetric(BaseMetric):
     async def _a_generate_verdicts(
         self, input: str, context: List[str]
     ) -> ContextualRelevancyVerdicts:
-        prompt = ContextualRelevancyTemplate.generate_verdicts(
+        prompt = self.evaluation_template.generate_verdicts(
             input=input, context=context
         )
         if self.using_native_model:
@@ -221,7 +240,7 @@ class ContextualRelevancyMetric(BaseMetric):
     def _generate_verdicts(
         self, input: str, context: str
     ) -> ContextualRelevancyVerdicts:
-        prompt = ContextualRelevancyTemplate.generate_verdicts(
+        prompt = self.evaluation_template.generate_verdicts(
             input=input, context=context
         )
         if self.using_native_model:
