@@ -1,16 +1,30 @@
-"""An implementation of the Ragas metric
-"""
+"""An implementation of the Ragas metric"""
+
+from typing import Optional, Union, List
 
-from typing import Optional, Union, Any, List
-from langchain_core.language_models import BaseChatModel
-from langchain_core.embeddings import Embeddings
-from enum import Enum
 
 from deepeval.metrics import BaseMetric
 from deepeval.test_case import LLMTestCase
 from deepeval.models import GPTModel
 from deepeval.telemetry import capture_metric_type
 
+# check langchain availability
+try:
+    import langchain_core
+    from langchain_core.language_models import BaseChatModel
+    from langchain_core.embeddings import Embeddings
+
+    langchain_available = True
+except ImportError:
+    langchain_available = False
+
+
+def _check_langchain_available():
+    if not langchain_available:
+        raise ImportError(
+            "langchain_core is required for this functionality. Install it via your package manager"
+        )
+
 
 def format_ragas_metric_name(name: str):
     return f"{name} (ragas)"
@@ -27,9 +41,10 @@ class RAGASContextualPrecisionMetric(BaseMetric):
     def __init__(
         self,
         threshold: float = 0.3,
-        model: Optional[Union[str, BaseChatModel]] = "gpt-3.5-turbo",
+        model: Optional[Union[str, "BaseChatModel"]] = "gpt-3.5-turbo",
         _track: bool = True,
     ):
+        _check_langchain_available()
         self.threshold = threshold
         self.model = model
         self._track = _track
@@ -66,7 +81,9 @@ class RAGASContextualPrecisionMetric(BaseMetric):
         }
         dataset = Dataset.from_dict(data)
 
-        with capture_metric_type(self.__name__, _track=self._track):
+        with capture_metric_type(
+            self.__name__, _track=self._track, async_mode=False
+        ):
             # Evaluate the dataset using Ragas
             scores = evaluate(
                 dataset, metrics=[context_precision], llm=chat_model
@@ -102,7 +119,7 @@ class RAGASContextualRecallMetric(BaseMetric):
     def __init__(
         self,
         threshold: float = 0.3,
-        model: Optional[Union[str, BaseChatModel]] = "gpt-3.5-turbo",
+        model: Optional[Union[str, "BaseChatModel"]] = "gpt-3.5-turbo",
         _track: bool = True,
     ):
         self.threshold = threshold
@@ -144,7 +161,9 @@ class RAGASContextualRecallMetric(BaseMetric):
             "contexts": [test_case.retrieval_context],
         }
         dataset = Dataset.from_dict(data)
-        with capture_metric_type(self.__name__, _track=self._track):
+        with capture_metric_type(
+            self.__name__, _track=self._track, async_mode=False
+        ):
             scores = evaluate(dataset, [context_recall], llm=chat_model)
             context_recall_score = scores["context_recall"][0]
             self.success = context_recall_score >= self.threshold
@@ -170,7 +189,7 @@ class RAGASContextualEntitiesRecall(BaseMetric):
     def __init__(
         self,
         threshold: float = 0.3,
-        model: Optional[Union[str, BaseChatModel]] = "gpt-3.5-turbo",
+        model: Optional[Union[str, "BaseChatModel"]] = "gpt-3.5-turbo",
         _track: bool = True,
     ):
         self.threshold = threshold
@@ -213,7 +232,9 @@ class RAGASContextualEntitiesRecall(BaseMetric):
         }
         dataset = Dataset.from_dict(data)
 
-        with capture_metric_type(self.__name__, _track=self._track):
+        with capture_metric_type(
+            self.__name__, _track=self._track, async_mode=False
+        ):
             scores = evaluate(
                 dataset,
                 metrics=[ContextEntityRecall()],
@@ -237,7 +258,7 @@ class RAGASContextualEntitiesRecall(BaseMetric):
 #############################################################
 
 # class RAGASNoiseSensitivityMetric(BaseMetric):
-#     """This metric checks the noise sentivity using Ragas"""
+#     """This metric checks the noise sensitivity using Ragas"""
 
 #     def __init__(
 #         self,
@@ -291,9 +312,9 @@ class RAGASContextualEntitiesRecall(BaseMetric):
 #                 metrics=[NoiseSensitivity()],
 #                 llm=chat_model,
 #             )
-#             noise_sensitivty_score = scores["noise_sensitivity_relevant"][0]
-#             self.success = noise_sensitivty_score >= self.threshold
-#             self.score = noise_sensitivty_score
+#             noise_sensitivity_score = scores["noise_sensitivity_relevant"][0]
+#             self.success = noise_sensitivity_score >= self.threshold
+#             self.score = noise_sensitivity_score
 #             return self.score
 
 #     def is_successful(self):
@@ -315,8 +336,8 @@ class RAGASAnswerRelevancyMetric(BaseMetric):
     def __init__(
         self,
         threshold: float = 0.3,
-        model: Optional[Union[str, BaseChatModel]] = "gpt-3.5-turbo",
-        embeddings: Optional[Embeddings] = None,
+        model: Optional[Union[str, "BaseChatModel"]] = "gpt-3.5-turbo",
+        embeddings: Optional["Embeddings"] = None,
         _track: bool = True,
     ):
         self.threshold = threshold
@@ -361,7 +382,9 @@ class RAGASAnswerRelevancyMetric(BaseMetric):
         }
         dataset = Dataset.from_dict(data)
 
-        with capture_metric_type(self.__name__, _track=self._track):
+        with capture_metric_type(
+            self.__name__, _track=self._track, async_mode=False
+        ):
             scores = evaluate(
                 dataset,
                 metrics=[ResponseRelevancy(embeddings=self.embeddings)],
@@ -390,7 +413,7 @@ class RAGASFaithfulnessMetric(BaseMetric):
     def __init__(
         self,
         threshold: float = 0.3,
-        model: Optional[Union[str, BaseChatModel]] = "gpt-3.5-turbo",
+        model: Optional[Union[str, "BaseChatModel"]] = "gpt-3.5-turbo",
         _track: bool = True,
     ):
         self.threshold = threshold
@@ -433,7 +456,9 @@ class RAGASFaithfulnessMetric(BaseMetric):
             "answer": [test_case.actual_output],
         }
         dataset = Dataset.from_dict(data)
-        with capture_metric_type(self.__name__, _track=self._track):
+        with capture_metric_type(
+            self.__name__, _track=self._track, async_mode=False
+        ):
             scores = evaluate(dataset, metrics=[faithfulness], llm=chat_model)
             faithfulness_score = scores["faithfulness"][0]
             self.success = faithfulness_score >= self.threshold
@@ -459,8 +484,8 @@ class RagasMetric(BaseMetric):
     def __init__(
         self,
         threshold: float = 0.3,
-        model: Optional[Union[str, BaseChatModel]] = "gpt-3.5-turbo",
-        embeddings: Optional[Embeddings] = None,
+        model: Optional[Union[str, "BaseChatModel"]] = "gpt-3.5-turbo",
+        embeddings: Optional["Embeddings"] = None,
     ):
         self.threshold = threshold
         self.model = model
@@ -501,7 +526,7 @@ class RagasMetric(BaseMetric):
             RAGASFaithfulnessMetric(model=self.model, _track=False),
         ]
 
-        with capture_metric_type(self.__name__):
+        with capture_metric_type(self.__name__, async_mode=False):
             for metric in metrics:
                 score = metric.measure(test_case)
                 score_breakdown[metric.__name__] = score
