@@ -1,6 +1,5 @@
 import pytest
 import os
-import json
 from rich import print
 from typing import Optional, Any
 from deepeval.constants import PYTEST_RUN_TEST_NAME
@@ -8,13 +7,24 @@ from deepeval.test_run import global_test_run_manager
 from deepeval.utils import get_is_running_deepeval
 
 
+def pytest_addoption(parser):
+    parser.addoption(
+        "--identifier",
+        action="store",
+        default=None,
+        help="Custom identifier for the test run",
+    )
+
+
 def pytest_sessionstart(session: pytest.Session):
     is_running_deepeval = get_is_running_deepeval()
+    identifier = session.config.getoption("identifier", None)
 
     if is_running_deepeval:
         global_test_run_manager.save_to_disk = True
         global_test_run_manager.create_test_run(
-            file_name=session.config.getoption("file_or_dir")[0]
+            identifier=identifier,
+            file_name=session.config.getoption("file_or_dir")[0],
         )
 
 
