@@ -1,3 +1,4 @@
+from enum import Enum
 import os
 import json
 from pydantic import BaseModel, Field
@@ -12,22 +13,35 @@ from rich.console import Console
 from rich import print
 
 from deepeval.metrics import BaseMetric
-from deepeval.confident.api import Api, Endpoints, HttpMethods
+from deepeval.confident.api import Api, Endpoints, HttpMethods, is_confident
 from deepeval.test_run.api import (
     LLMApiTestCase,
     ConversationalApiTestCase,
     TestRunHttpResponse,
+    MetricData,
 )
+from deepeval.tracing.utils import make_json_serializable
+from deepeval.tracing.api import SpanApiType, span_api_type_literals
 from deepeval.test_case import LLMTestCase, ConversationalTestCase, MLLMTestCase
 from deepeval.utils import (
     delete_file_if_exists,
     get_is_running_deepeval,
-    is_confident,
-    is_in_ci_env,
+    open_browser,
 )
 from deepeval.test_run.cache import global_test_run_cache_manager
+from deepeval.constants import CONFIDENT_TEST_CASE_BATCH_SIZE, HIDDEN_DIR
 
-TEMP_FILE_NAME = "temp_test_run_data.json"
+TEMP_FILE_PATH = f"{HIDDEN_DIR}/.temp_test_run_data.json"
+LATEST_TEST_RUN_FILE_PATH = f"{HIDDEN_DIR}/.latest_test_run.json"
+LATEST_TEST_RUN_DATA_KEY = "testRunData"
+LATEST_TEST_RUN_LINK_KEY = "testRunLink"
+console = Console()
+
+
+class TestRunResultDisplay(Enum):
+    ALL = "all"
+    FAILING = "failing"
+    PASSING = "passing"
 
 
 class MetricScoreType(BaseModel):
@@ -42,6 +56,17 @@ class MetricScoreType(BaseModel):
 class MetricScores(BaseModel):
     metric: str
     scores: List[float]
+    passes: int
+    fails: int
+    errors: int
+
+
+class TraceMetricScores(BaseModel):
+    agent: Dict[str, Dict[str, MetricScores]] = Field(default_factory=dict)
+    tool: Dict[str, Dict[str, MetricScores]] = Field(default_factory=dict)
+    retriever: Dict[str, Dict[str, MetricScores]] = Field(default_factory=dict)
+    llm: Dict[str, Dict[str, MetricScores]] = Field(default_factory=dict)
+    base: Dict[str, Dict[str, MetricScores]] = Field(default_factory=dict)
 
 
 class MetricsAverageDict:
@@ -91,8 +116,11 @@ class TestRun(BaseModel):
     metrics_scores: List[MetricScores] = Field(
         default_factory=lambda: [], alias="metricsScores"
     )
+    trace_metrics_scores: Optional[TraceMetricScores] = Field(
+        None, alias="traceMetricsScores"
+    )
     identifier: Optional[str] = None
-    hyperparameters: Optional[Dict[Any, Any]] = Field(None)
+    hyperparameters: Optional[Dict[str, Any]] = Field(None)
     test_passed: Optional[int] = Field(None, alias="testPassed")
     test_failed: Optional[int] = Field(None, alias="testFailed")
     run_duration: float = Field(0.0, alias="runDuration")
@@ -106,36 +134,7 @@ class TestRun(BaseModel):
         if isinstance(api_test_case, ConversationalApiTestCase):
             self.conversational_test_cases.append(api_test_case)
         else:
-            if api_test_case.conversational_instance_id is not None:
-                for conversational_test_case in self.conversational_test_cases:
-                    if (
-                        api_test_case.conversational_instance_id
-                        == conversational_test_case.instance_id
-                    ):
-                        conversational_test_case.turns[api_test_case.order] = (
-                            api_test_case
-                        )
-
-                        if api_test_case.success is False:
-                            conversational_test_case.success = False
-
-                        if conversational_test_case.evaluation_cost is None:
-                            conversational_test_case.evaluation_cost = (
-                                api_test_case.evaluation_cost
-                            )
-                        else:
-                            if api_test_case.evaluation_cost is not None:
-                                conversational_test_case.evaluation_cost += (
-                                    api_test_case.evaluation_cost
-                                )
-
-                        conversational_test_case.run_duration += (
-                            api_test_case.run_duration
-                        )
-                        break
-
-            else:
-                self.test_cases.append(api_test_case)
+            self.test_cases.append(api_test_case)
 
         if api_test_case.evaluation_cost is not None:
             if self.evaluation_cost is None:
@@ -174,65 +173,179 @@ class TestRun(BaseModel):
                 test_case.order = highest_order
             highest_order = test_case.order + 1
 
-    def delete_test_case_instance_ids(self):
-        for conversational_test_case in self.conversational_test_cases:
-            del conversational_test_case.instance_id
-            for turn in conversational_test_case.turns:
-                del turn.conversational_instance_id
-
-        for test_case in self.test_cases:
-            del test_case.conversational_instance_id
-
     def construct_metrics_scores(self) -> int:
-        metrics_dict: Dict[str, List[float]] = {}
+        # Use a dict to aggregate scores, passes, and fails for each metric.
+        metrics_dict: Dict[str, Dict[str, Any]] = {}
+        # Add dict for trace metrics
+        trace_metrics_dict: Dict[
+            span_api_type_literals, Dict[str, Dict[str, Dict[str, Any]]]
+        ] = {
+            SpanApiType.AGENT.value: {},
+            SpanApiType.TOOL.value: {},
+            SpanApiType.RETRIEVER.value: {},
+            SpanApiType.LLM.value: {},
+            SpanApiType.BASE.value: {},
+        }
         valid_scores = 0
+
+        def process_metric_data(metric_data: MetricData):
+            nonlocal valid_scores
+            name = metric_data.name
+            score = metric_data.score
+            success = metric_data.success
+            # Initialize dict entry if needed.
+            if name not in metrics_dict:
+                metrics_dict[name] = {
+                    "scores": [],
+                    "passes": 0,
+                    "fails": 0,
+                    "errors": 0,
+                }
+
+            if score is None or success is None:
+                metrics_dict[name]["errors"] += 1
+            else:
+                valid_scores += 1
+
+                # Append the score.
+                metrics_dict[name]["scores"].append(score)
+
+                # Increment passes or fails based on the metric_data.success flag.
+                if success:
+                    metrics_dict[name]["passes"] += 1
+                else:
+                    metrics_dict[name]["fails"] += 1
+
+        def process_span_metric_data(
+            metric_data: MetricData, type: span_api_type_literals, name: str
+        ):
+            metric_name = metric_data.name
+            score = metric_data.score
+            success = metric_data.success
+
+            # Initialize the structure if needed
+            if name not in trace_metrics_dict[type]:
+                trace_metrics_dict[type][name] = {}
+
+            if metric_name not in trace_metrics_dict[type][name]:
+                trace_metrics_dict[type][name][metric_name] = {
+                    "scores": [],
+                    "passes": 0,
+                    "fails": 0,
+                    "errors": 0,
+                }
+
+            if score is None or success is None:
+                trace_metrics_dict[type][name][metric_name]["errors"] += 1
+            else:
+                # Append the score
+                trace_metrics_dict[type][name][metric_name]["scores"].append(
+                    score
+                )
+
+                # Increment passes or fails
+                if success:
+                    trace_metrics_dict[type][name][metric_name]["passes"] += 1
+                else:
+                    trace_metrics_dict[type][name][metric_name]["fails"] += 1
+
+        # Process non-conversational test cases.
         for test_case in self.test_cases:
             if test_case.metrics_data is None:
                 continue
             for metric_data in test_case.metrics_data:
-                name = metric_data.name
-                score = metric_data.score
-                if score is None:
-                    continue
-                valid_scores += 1
-                if name in metrics_dict:
-                    metrics_dict[name].append(score)
-                else:
-                    metrics_dict[name] = [score]
+                process_metric_data(metric_data)
 
+            if test_case.trace is None:
+                continue
+
+            for span in test_case.trace.agent_spans:
+                if span.metrics_data is not None:
+                    for metric_data in span.metrics_data:
+                        process_metric_data(metric_data)
+                        process_span_metric_data(
+                            metric_data, SpanApiType.AGENT.value, span.name
+                        )
+
+            for span in test_case.trace.tool_spans:
+                if span.metrics_data is not None:
+                    for metric_data in span.metrics_data:
+                        process_metric_data(metric_data)
+                        process_span_metric_data(
+                            metric_data, SpanApiType.TOOL.value, span.name
+                        )
+
+            for span in test_case.trace.retriever_spans:
+                if span.metrics_data is not None:
+                    for metric_data in span.metrics_data:
+                        process_metric_data(metric_data)
+                        process_span_metric_data(
+                            metric_data, SpanApiType.RETRIEVER.value, span.name
+                        )
+
+            for span in test_case.trace.llm_spans:
+                if span.metrics_data is not None:
+                    for metric_data in span.metrics_data:
+                        process_metric_data(metric_data)
+                        process_span_metric_data(
+                            metric_data, SpanApiType.LLM.value, span.name
+                        )
+
+            for span in test_case.trace.base_spans:
+                if span.metrics_data is not None:
+                    for metric_data in span.metrics_data:
+                        process_metric_data(metric_data)
+                        process_span_metric_data(
+                            metric_data, SpanApiType.BASE.value, span.name
+                        )
+
+        # Process conversational test cases.
         for convo_test_case in self.conversational_test_cases:
             if convo_test_case.metrics_data is not None:
                 for metric_data in convo_test_case.metrics_data:
-                    name = metric_data.name
-                    score = metric_data.score
-                    if score is None:
-                        continue
-                    valid_scores += 1
-                    if name in metrics_dict:
-                        metrics_dict[name].append(score)
-                    else:
-                        metrics_dict[name] = [score]
+                    process_metric_data(metric_data)
 
-            for turn in convo_test_case.turns:
-                if turn.metrics_data is None:
-                    continue
-                for metric_data in turn.metrics_data:
-                    name = metric_data.name
-                    score = metric_data.score
-                    if score is None:
-                        continue
-                    valid_scores += 1
-                    if name in metrics_dict:
-                        metrics_dict[name].append(score)
-                    else:
-                        metrics_dict[name] = [score]
-
-        # metrics_scores combines both conversational and nonconvo and mllm scores
-        # might need to separate in the future
+        # Create MetricScores objects with the aggregated data.
         self.metrics_scores = [
-            MetricScores(metric=metric, scores=scores)
-            for metric, scores in metrics_dict.items()
+            MetricScores(
+                metric=metric,
+                scores=data["scores"],
+                passes=data["passes"],
+                fails=data["fails"],
+                errors=data["errors"],
+            )
+            for metric, data in metrics_dict.items()
         ]
+
+        # Create a single TraceMetricScores object instead of a list
+        trace_metrics_score = TraceMetricScores()
+        has_span_metrics = False
+
+        for span_type, spans in trace_metrics_dict.items():
+            if not spans:  # Skip empty span types
+                continue
+
+            span_dict = {}
+            for span_name, metrics in spans.items():
+                span_dict[span_name] = {
+                    metric_name: MetricScores(
+                        metric=metric_name,
+                        scores=metric_data["scores"],
+                        passes=metric_data["passes"],
+                        fails=metric_data["fails"],
+                        errors=metric_data["errors"],
+                    )
+                    for metric_name, metric_data in metrics.items()
+                }
+
+            if span_dict:  # Only set if there are spans
+                has_span_metrics = True
+                setattr(trace_metrics_score, span_type, span_dict)
+
+        # Set to None if no span metrics were found
+        self.trace_metrics_scores = (
+            trace_metrics_score if has_span_metrics else None
+        )
         return valid_scores
 
     def calculate_test_passes_and_fails(self):
@@ -262,7 +375,7 @@ class TestRun(BaseModel):
         except AttributeError:
             # Pydantic version below 2.0
             body = self.dict(by_alias=True, exclude_none=True)
-        json.dump(body, f)
+        json.dump(body, f, cls=TestRunEncoder)
         return self
 
     @classmethod
@@ -278,16 +391,23 @@ class TestRun(BaseModel):
                 )
 
 
+class TestRunEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, Enum):
+            return obj.value
+        return make_json_serializable(obj)
+
+
 class TestRunManager:
     def __init__(self):
         self.test_run = None
-        self.temp_file_name = TEMP_FILE_NAME
+        self.temp_file_path = TEMP_FILE_PATH
         self.save_to_disk = False
         self.disable_request = False
 
     def reset(self):
         self.test_run = None
-        self.temp_file_name = TEMP_FILE_NAME
+        self.temp_file_path = TEMP_FILE_PATH
         self.save_to_disk = False
         self.disable_request = False
 
@@ -313,7 +433,7 @@ class TestRunManager:
         self.set_test_run(test_run)
 
         if self.save_to_disk:
-            self.save_test_run()
+            self.save_test_run(self.temp_file_path)
 
     def get_test_run(self, identifier: Optional[str] = None):
         if self.test_run is None:
@@ -322,7 +442,7 @@ class TestRunManager:
         if self.save_to_disk:
             try:
                 with portalocker.Lock(
-                    self.temp_file_name,
+                    self.temp_file_path,
                     mode="r",
                     flags=portalocker.LOCK_SH | portalocker.LOCK_NB,
                 ) as file:
@@ -336,16 +456,33 @@ class TestRunManager:
 
         return self.test_run
 
-    def save_test_run(self):
+    def save_test_run(self, path: str, save_under_key: Optional[str] = None):
         if self.save_to_disk:
             try:
-                with portalocker.Lock(self.temp_file_name, mode="w") as file:
-                    self.test_run = self.test_run.save(file)
+                with portalocker.Lock(path, mode="w") as file:
+                    if save_under_key:
+                        try:
+                            test_run_data = self.test_run.model_dump(
+                                by_alias=True, exclude_none=True
+                            )
+                        except AttributeError:
+                            # Pydantic version below 2.0
+                            test_run_data = self.test_run.dict(
+                                by_alias=True, exclude_none=True
+                            )
+                        wrapper_data = {save_under_key: test_run_data}
+                        json.dump(wrapper_data, file, cls=TestRunEncoder)
+                    else:
+                        self.test_run.save(file)
             except portalocker.exceptions.LockException:
-                print(
-                    "In save_test_run, Error saving test run to disk",
-                    file=sys.stderr,
-                )
+                pass
+
+    def save_final_test_run_link(self, link: str):
+        try:
+            with portalocker.Lock(LATEST_TEST_RUN_FILE_PATH, mode="w") as file:
+                json.dump({LATEST_TEST_RUN_LINK_KEY: link}, file)
+        except portalocker.exceptions.LockException:
+            pass
 
     def update_test_run(
         self,
@@ -355,13 +492,14 @@ class TestRunManager:
         if (
             api_test_case.metrics_data is not None
             and len(api_test_case.metrics_data) == 0
+            and api_test_case.trace is None
         ):
             return
 
         if self.save_to_disk:
             try:
                 with portalocker.Lock(
-                    self.temp_file_name,
+                    self.temp_file_path,
                     mode="r+",
                     flags=portalocker.LOCK_EX,
                 ) as file:
@@ -392,7 +530,9 @@ class TestRunManager:
     def clear_test_run(self):
         self.test_run = None
 
-    def display_results_table(self, test_run: TestRun):
+    def display_results_table(
+        self, test_run: TestRun, display: TestRunResultDisplay
+    ):
         table = Table(title="Test Results")
         table.add_column("Test case", justify="left")
         table.add_column("Metric", justify="left")
@@ -404,22 +544,39 @@ class TestRunManager:
             if test_case.metrics_data is None:
                 continue
 
+            if (
+                display == TestRunResultDisplay.PASSING
+                and test_case.success == False
+            ):
+                continue
+            elif display == TestRunResultDisplay.FAILING and test_case.success:
+                continue
+
             pass_count = 0
             fail_count = 0
             test_case_name = test_case.name
 
+            # TODO: recursively iterate through it to calculate pass and fail count
+            if test_case.trace:
+                pass
+
             for metric_data in test_case.metrics_data:
                 if metric_data.success:
                     pass_count += 1
                 else:
                     fail_count += 1
 
+            success_rate = (
+                round((100 * pass_count) / (pass_count + fail_count), 2)
+                if pass_count + fail_count > 0
+                else "Cannot display metrics for component-level evals, please run 'deepeval view' to see results on Confident AI."
+            )
             table.add_row(
                 test_case_name,
                 "",
                 "",
                 "",
-                f"{round((100*pass_count)/(pass_count+fail_count),2)}%",
+                f"{success_rate}%",
             )
 
             for metric_data in test_case.metrics_data:
@@ -459,6 +616,17 @@ class TestRunManager:
         for index, conversational_test_case in enumerate(
             test_run.conversational_test_cases
         ):
+            if (
+                display == TestRunResultDisplay.PASSING
+                and conversational_test_case.success == False
+            ):
+                continue
+            elif (
+                display == TestRunResultDisplay.FAILING
+                and conversational_test_case.success
+            ):
+                continue
+
             pass_count = 0
             fail_count = 0
             conversational_test_case_name = conversational_test_case.name
@@ -503,54 +671,6 @@ class TestRunManager:
                         "",
                     )
 
-            for turn in conversational_test_case.turns:
-                if turn.metrics_data is None:
-                    # skip if no evaluation
-                    continue
-
-                pass_count = 0
-                fail_count = 0
-                test_case_name = test_case.name
-
-                for metric_data in test_case.metrics_data:
-                    if metric_data.success:
-                        pass_count += 1
-                    else:
-                        fail_count += 1
-
-                table.add_row(
-                    test_case_name,
-                    "",
-                    "",
-                    "",
-                    f"{round((100*pass_count)/(pass_count+fail_count),2)}%",
-                )
-
-                for metric_data in test_case.metrics_data:
-                    if metric_data.error:
-                        status = "[red]ERRORED[/red]"
-                    elif metric_data.success:
-                        status = "[green]PASSED[/green]"
-                    else:
-                        status = "[red]FAILED[/red]"
-
-                    evaluation_model = metric_data.evaluation_model
-                    if evaluation_model is None:
-                        evaluation_model = "n/a"
-
-                    if metric_data.score is not None:
-                        metric_score = round(metric_data.score, 2)
-                    else:
-                        metric_score = None
-
-                    table.add_row(
-                        "",
-                        str(metric_data.name),
-                        f"{metric_score} (threshold={metric_data.threshold}, evaluation model={evaluation_model}, reason={metric_data.reason}, error={metric_data.error})",
-                        status,
-                        "",
-                    )
-
             if index is not len(self.test_run.conversational_test_cases) - 1:
                 table.add_row(
                     "",
@@ -569,129 +689,132 @@ class TestRunManager:
                     "",
                 )
 
-        print(table)
-        print(
-            f"Total estimated evaluation tokens cost: {test_run.evaluation_cost} USD"
+        table.add_row(
+            "[bold red]Note: Use Confident AI with DeepEval to analyze failed test cases for more details[/bold red]",
+            "",
+            "",
+            "",
+            "",
         )
+        print(table)
 
     def post_test_run(self, test_run: TestRun) -> Optional[str]:
-        console = Console()
-        if is_confident() and self.disable_request is False:
-            BATCH_SIZE = 60
-            CONVERSATIONAL_BATCH_SIZE = BATCH_SIZE // 3
+        if (
+            len(test_run.test_cases) == 0
+            and len(test_run.conversational_test_cases) == 0
+        ):
+            print("No test cases found, unable to upload to Confident AI.")
+            return
 
-            initial_batch = test_run.test_cases[:BATCH_SIZE]
-            remaining_test_cases = test_run.test_cases[BATCH_SIZE:]
+        api = Api()
 
-            initial_conversational_batch = test_run.conversational_test_cases[
-                :CONVERSATIONAL_BATCH_SIZE
-            ]
-            remaining_conversational_test_cases = (
-                test_run.conversational_test_cases[CONVERSATIONAL_BATCH_SIZE:]
+        is_conversational_run = len(test_run.conversational_test_cases) > 0
+        all_test_cases_to_process = (
+            test_run.conversational_test_cases
+            if is_conversational_run
+            else test_run.test_cases
+        )
+
+        custom_batch_size = os.getenv(CONFIDENT_TEST_CASE_BATCH_SIZE)
+        if custom_batch_size and custom_batch_size.isdigit():
+            BATCH_SIZE = int(custom_batch_size)
+        else:
+            BATCH_SIZE = 20 if is_conversational_run else 40
+
+        initial_batch = all_test_cases_to_process[:BATCH_SIZE]
+        remaining_test_cases_to_process = all_test_cases_to_process[BATCH_SIZE:]
+
+        if len(remaining_test_cases_to_process) > 0:
+            console.print(
+                "Sending a large test run to Confident, this might take a bit longer than usual..."
             )
 
-            if (
-                len(remaining_test_cases) > 0
-                or len(remaining_conversational_test_cases) > 0
-            ):
-                console.print(
-                    "Sending a large test run to Confident, this might take a bit longer than usual..."
+        ####################
+        ### POST REQUEST ###
+        ####################
+        if is_conversational_run:
+            test_run.conversational_test_cases = initial_batch
+        else:
+            test_run.test_cases = initial_batch
+
+        try:
+            body = test_run.model_dump(by_alias=True, exclude_none=True)
+        except AttributeError:
+            # Pydantic version below 2.0
+            body = test_run.dict(by_alias=True, exclude_none=True)
+
+        json_str = json.dumps(body, cls=TestRunEncoder)
+        body = json.loads(json_str)
+
+        data, link = api.send_request(
+            method=HttpMethods.POST,
+            endpoint=Endpoints.TEST_RUN_ENDPOINT,
+            body=body,
+        )
+
+        res = TestRunHttpResponse(
+            id=data["id"],
+        )
+
+        ################################################
+        ### Send the remaining test cases in batches ###
+        ################################################
+        total_remaining = len(remaining_test_cases_to_process)
+        num_remaining_batches = (
+            (total_remaining + BATCH_SIZE - 1) // BATCH_SIZE
+            if total_remaining > 0
+            else 0
+        )
+
+        for i in range(num_remaining_batches):
+            start_index = i * BATCH_SIZE
+            batch = remaining_test_cases_to_process[
+                start_index : start_index + BATCH_SIZE
+            ]
+
+            if len(batch) == 0:
+                break  # Should not happen with correct num_remaining_batches, but as a safeguard
+
+            # Create RemainingTestRun with the correct list populated
+            if is_conversational_run:
+                remaining_test_run = RemainingTestRun(
+                    testRunId=res.id,
+                    testCases=[],  # This will be empty
+                    conversationalTestCases=batch,
+                )
+            else:
+                remaining_test_run = RemainingTestRun(
+                    testRunId=res.id,
+                    testCases=batch,
+                    conversationalTestCases=[],  # This will be empty
                 )
 
-            ####################
-            ### POST REQUEST ###
-            ####################
-            test_run.test_cases = initial_batch
-            test_run.conversational_test_cases = initial_conversational_batch
+            body = None
             try:
-                body = test_run.model_dump(by_alias=True, exclude_none=True)
+                body = remaining_test_run.model_dump(
+                    by_alias=True, exclude_none=True
+                )
             except AttributeError:
                 # Pydantic version below 2.0
-                body = test_run.dict(by_alias=True, exclude_none=True)
+                body = remaining_test_run.dict(by_alias=True, exclude_none=True)
 
-            api = Api()
-            result = api.send_request(
-                method=HttpMethods.POST,
-                endpoint=Endpoints.TEST_RUN_ENDPOINT,
-                body=body,
-            )
-
-            response = TestRunHttpResponse(
-                testRunId=result["testRunId"],
-                projectId=result["projectId"],
-                link=result["link"],
-            )
-            link = response.link
-            ################################################
-            ### Send the remaining test cases in batches ###
-            ################################################
-            max_iterations = (
-                max(
-                    len(remaining_test_cases),
-                    len(remaining_conversational_test_cases),
+            try:
+                _, _ = api.send_request(
+                    method=HttpMethods.PUT,
+                    endpoint=Endpoints.TEST_RUN_ENDPOINT,
+                    body=body,
                 )
-                // CONVERSATIONAL_BATCH_SIZE
-            )
-            for i in range(0, max_iterations + 1):
-                test_case_index = (
-                    i * CONVERSATIONAL_BATCH_SIZE * 3
-                )  # Multiply by 3 to match the conversational batch step
-                test_case_batch = remaining_test_cases[
-                    test_case_index : test_case_index + BATCH_SIZE
-                ]
+            except Exception as e:
+                message = f"Unexpected error when sending some test cases. Incomplete test run available at {link}"
+                raise Exception(message) from e
 
-                # Adjusting for conversational_test_cases
-                conversational_index = i * CONVERSATIONAL_BATCH_SIZE
-                conversational_batch = remaining_conversational_test_cases[
-                    conversational_index : conversational_index
-                    + CONVERSATIONAL_BATCH_SIZE
-                ]
-
-                if len(test_case_batch) == 0 and len(conversational_batch) == 0:
-                    break
-
-                remaining_test_run = RemainingTestRun(
-                    testRunId=response.testRunId,
-                    testCases=test_case_batch,
-                    conversationalTestCases=conversational_batch,
-                )
-
-                body = None
-                try:
-                    body = remaining_test_run.model_dump(
-                        by_alias=True, exclude_none=True
-                    )
-                except AttributeError:
-                    # Pydantic version below 2.0
-                    body = remaining_test_run.dict(
-                        by_alias=True, exclude_none=True
-                    )
-
-                try:
-                    result = api.send_request(
-                        method=HttpMethods.PUT,
-                        endpoint=Endpoints.TEST_RUN_ENDPOINT,
-                        body=body,
-                    )
-                except Exception as e:
-                    message = f"Unexpected error when sending some test cases. Incomplete test run available at {link}"
-                    raise Exception(message) from e
-
-            console.print(
-                "[rgb(5,245,141)]✓[/rgb(5,245,141)] Tests finished 🎉! View results on "
-                f"[link={link}]{link}[/link]."
-                "\n‼️  Friendly reminder 😇: You can also run evaluations with ALL of deepeval's metrics directly on Confident AI instead."
-            )
-
-            if is_in_ci_env() == False:
-                webbrowser.open(link)
-
-            return link
-        else:
-            console.print(
-                "[rgb(5,245,141)]✓[/rgb(5,245,141)] Tests finished 🎉! Run 'deepeval login' to save and analyze evaluation results on Confident AI. "
-                "\n‼️  Friendly reminder 😇: You can also run evaluations with ALL of deepeval's metrics directly on Confident AI instead."
-            )
+        console.print(
+            "[rgb(5,245,141)]✓[/rgb(5,245,141)] Done 🎉! View results on "
+            f"[link={link}]{link}[/link]"
+        )
+        self.save_final_test_run_link(link)
+        open_browser(link)
+        return link
 
     def save_test_run_locally(self):
         local_folder = os.getenv("DEEPEVAL_RESULTS_FOLDER")
@@ -699,7 +822,7 @@ class TestRunManager:
             new_test_filename = datetime.datetime.now().strftime(
                 "%Y%m%d_%H%M%S"
             )
-            os.rename(self.temp_file_name, new_test_filename)
+            os.rename(self.temp_file_path, new_test_filename)
             if not os.path.exists(local_folder):
                 os.mkdir(local_folder)
                 shutil.copy(new_test_filename, local_folder)
@@ -714,25 +837,28 @@ class TestRunManager:
             os.remove(new_test_filename)
 
     def wrap_up_test_run(
-        self, runDuration: float, display_table: bool = True
+        self,
+        runDuration: float,
+        display_table: bool = True,
+        display: Optional[TestRunResultDisplay] = TestRunResultDisplay.ALL,
     ) -> Optional[str]:
         test_run = self.get_test_run()
         if test_run is None:
             print("Test Run is empty, please try again.")
-            delete_file_if_exists(self.temp_file_name)
+            delete_file_if_exists(self.temp_file_path)
             return
         elif (
             len(test_run.test_cases) == 0
             and len(test_run.conversational_test_cases) == 0
         ):
             print("No test cases found, please try again.")
-            delete_file_if_exists(self.temp_file_name)
+            delete_file_if_exists(self.temp_file_path)
             return
 
         valid_scores = test_run.construct_metrics_scores()
         if valid_scores == 0:
             print("All metrics errored for all test cases, please try again.")
-            delete_file_if_exists(self.temp_file_name)
+            delete_file_if_exists(self.temp_file_path)
             delete_file_if_exists(
                 global_test_run_cache_manager.temp_cache_file_name
             )
@@ -740,7 +866,6 @@ class TestRunManager:
         test_run.run_duration = runDuration
         test_run.calculate_test_passes_and_fails()
         test_run.sort_test_cases()
-        test_run.delete_test_case_instance_ids()
 
         if global_test_run_cache_manager.disable_write_cache is None:
             global_test_run_cache_manager.disable_write_cache = (
@@ -750,17 +875,47 @@ class TestRunManager:
         global_test_run_cache_manager.wrap_up_cached_test_run()
 
         if display_table:
-            self.display_results_table(test_run)
+            self.display_results_table(test_run, display)
 
         self.save_test_run_locally()
-        delete_file_if_exists(self.temp_file_name)
-
-        if (
-            len(test_run.test_cases) > 0
-            or len(test_run.conversational_test_cases) > 0
-        ):
-            test_run.guard_mllm_test_cases()
+        delete_file_if_exists(self.temp_file_path)
+        if is_confident() and self.disable_request is False:
             return self.post_test_run(test_run)
+        else:
+            self.save_test_run(
+                LATEST_TEST_RUN_FILE_PATH,
+                save_under_key=LATEST_TEST_RUN_DATA_KEY,
+            )
+            console.print(
+                f"\n\n[rgb(5,245,141)]✓[/rgb(5,245,141)] Evaluation completed 🎉! (time taken: {round(runDuration, 2)}s | token cost: {test_run.evaluation_cost} USD)\n"
+                f"» Test Results ({test_run.test_passed + test_run.test_failed} total tests):\n",
+                f"  » Pass Rate: {round((test_run.test_passed / (test_run.test_passed + test_run.test_failed)) * 100, 2)}% | Passed: [bold green]{test_run.test_passed}[/bold green] | Failed: [bold red]{test_run.test_failed}[/bold red]\n\n",
+                "=" * 80,
+                "\n\n» What to share evals with your team, or a place for your test cases to live? ❤️ 🏡\n"
+                "  » Run [bold]'deepeval view'[/bold] to analyze and save testing results on [rgb(106,0,255)]Confident AI[/rgb(106,0,255)].\n\n",
+            )
+
+    def get_latest_test_run_data(self) -> Optional[TestRun]:
+        try:
+            if os.path.exists(LATEST_TEST_RUN_FILE_PATH):
+                with open(LATEST_TEST_RUN_FILE_PATH, "r") as file:
+                    data = json.load(file)
+                    return TestRun.model_validate(
+                        data[LATEST_TEST_RUN_DATA_KEY]
+                    )
+        except (FileNotFoundError, json.JSONDecodeError, Exception):
+            pass
+        return None
+
+    def get_latest_test_run_link(self) -> Optional[str]:
+        try:
+            if os.path.exists(LATEST_TEST_RUN_FILE_PATH):
+                with open(LATEST_TEST_RUN_FILE_PATH, "r") as file:
+                    data = json.load(file)
+                    return data[LATEST_TEST_RUN_LINK_KEY]
+        except (FileNotFoundError, json.JSONDecodeError, Exception):
+            pass
+        return None
 
 
 global_test_run_manager = TestRunManager()
